"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lexer_1 = require("../lexer");
var error_message_1 = require("./error-message");
var parse_result_1 = require("./parse-result");
var resolution_context_1 = require("./resolution-context");
var ParserBase = /** @class */ (function () {
    function ParserBase() {
    }
    ParserBase.prototype.createLexer = function (input) {
        if (this.isLexer(input)) {
            return input;
        }
        else if (typeof input === 'string') {
            return new lexer_1.DefaultLexer(input);
        }
        else {
            throw new Error('Unrecognized input type. input must be of type "ILexer | string".');
        }
    };
    ParserBase.prototype.createContext = function (context) {
        if (this.isResolutionContext(context)) {
            return context;
        }
        else {
            return new resolution_context_1.DefaultResolutionContext(context);
        }
    };
    ParserBase.prototype.isLexer = function (input) {
        return input.getNextToken;
    };
    ParserBase.prototype.isResolutionContext = function (input) {
        return input && input.resolve;
    };
    ParserBase.prototype.parse = function (input) {
        var _this = this;
        var lexer = this.createLexer(input);
        var errors = [];
        var exp = this.parseCore(lexer, errors);
        var wrapper = function (context) { return exp(_this.createContext(context)); };
        return new parse_result_1.ParseResult(wrapper, errors);
    };
    ParserBase.prototype.expectAndConsume = function (lexer, errors, expected, actual) {
        this.expect(lexer, errors, expected, actual);
        return lexer.getNextToken();
    };
    ParserBase.prototype.expect = function (lexer, errors, expected, actual) {
        actual = actual || lexer.peekNextToken();
        if (actual.type !== expected) {
            this.errorToken(errors, expected, actual);
        }
        return actual;
    };
    ParserBase.prototype.errorToken = function (errors, expected, actual) {
        var message = "Error at position " + actual.position + ".";
        message += " Expected token of type " + expected + ", found token of type " + actual.type + " of value '" + actual.value + "'.";
        this.errorMessage(errors, message, actual);
    };
    ParserBase.prototype.errorMessage = function (errors, message, token) {
        errors.push(new error_message_1.ErrorMessage(message, token, new Error().stack));
    };
    return ParserBase;
}());
exports.ParserBase = ParserBase;
//# sourceMappingURL=parser-base.js.map