"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var _a, _b, _c, _d;
var lexer_1 = require("../lexer");
var parser_base_1 = require("./parser-base");
var UnaryOperatorMap = (_a = {},
    _a[lexer_1.TokenType.Plus] = function (n) { return +n; },
    _a[lexer_1.TokenType.Minus] = function (n) { return -n; },
    _a[lexer_1.TokenType.Exclamation] = function (n) { return !n; },
    _a);
var AddOperatorMap = (_b = {},
    _b[lexer_1.TokenType.Plus] = function (l, r) { return l + r; },
    _b[lexer_1.TokenType.Minus] = function (l, r) { return l - r; },
    _b);
var MultiOperatorMap = (_c = {},
    _c[lexer_1.TokenType.Asterisk] = function (l, r) { return l * r; },
    _c[lexer_1.TokenType.Slash] = function (l, r) { return l / r; },
    _c[lexer_1.TokenType.Percent] = function (l, r) { return l % r; },
    _c);
var BooleanOperatorMap = (_d = {},
    _d[lexer_1.TokenType.Equals] = function (l, r) { return l === r; },
    _d[lexer_1.TokenType.LessThan] = function (l, r) { return l < r; },
    _d[lexer_1.TokenType.LessThanEquals] = function (l, r) { return l <= r; },
    _d[lexer_1.TokenType.GreaterThan] = function (l, r) { return l > r; },
    _d[lexer_1.TokenType.GreaterThanEquals] = function (l, r) { return l >= r; },
    _d[lexer_1.TokenType.Pipe] = function (l, r) { return !!(l || r); },
    _d[lexer_1.TokenType.Ampersand] = function (l, r) { return !!(l && r); },
    _d);
var Parser = /** @class */ (function (_super) {
    __extends(Parser, _super);
    function Parser() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Parser.prototype.parseCore = function (lexer, errors) {
        return this.parseExpression(lexer, errors);
    };
    ;
    Parser.prototype.parseExpression = function (lexer, errors) {
        var lhs = this.parseSimpleExpression(lexer, errors);
        var operator = BooleanOperatorMap[lexer.peekNextToken().type];
        if (!operator) {
            return lhs;
        }
        lexer.getNextToken();
        var rhs = this.parseSimpleExpression(lexer, errors);
        return function (s) { return operator(lhs(s), rhs(s)); };
    };
    Parser.prototype.parseSimpleExpression = function (lexer, errors) {
        var tokenType = lexer.peekNextToken().type;
        var unaryOp = UnaryOperatorMap[tokenType];
        // Consume unary operator. '+3' for example.
        if (unaryOp) {
            lexer.getNextToken();
        }
        // Parse as a term. Will be a number if we've just consumed the unary token.
        var root = this.parseTerm(lexer, errors);
        // If we had a unary operator, then add it in.
        if (unaryOp) {
            var n_1 = root;
            root = function (s) { return unaryOp(n_1(s)); };
        }
        tokenType = lexer.peekNextToken().type;
        var _loop_1 = function () {
            var operation = AddOperatorMap[tokenType];
            // Consume the operator.
            lexer.getNextToken();
            var l = root;
            var r = this_1.parseTerm(lexer, errors);
            root = function (s) { return operation(l(s), r(s)); };
            tokenType = lexer.peekNextToken().type;
        };
        var this_1 = this;
        while (Object.keys(AddOperatorMap).indexOf(tokenType) > -1) {
            _loop_1();
        }
        return root;
    };
    Parser.prototype.parseTerm = function (lexer, errors) {
        var root = this.parseFactor(lexer, errors);
        var tokenType = lexer.peekNextToken().type;
        var _loop_2 = function () {
            var operation = MultiOperatorMap[tokenType];
            // Consume the operator.
            lexer.getNextToken();
            var l = root;
            var r = this_2.parseFactor(lexer, errors);
            root = function (s) { return operation(l(s), r(s)); };
            tokenType = lexer.peekNextToken().type;
        };
        var this_2 = this;
        while (Object.keys(MultiOperatorMap).indexOf(tokenType) > -1) {
            _loop_2();
        }
        return root;
    };
    Parser.prototype.parseFactor = function (lexer, errors) {
        var token = lexer.peekNextToken();
        switch (token.type) {
            case lexer_1.TokenType.Identifier:
                var identifier = this.parseDottedIdentifier(lexer, errors);
                if (lexer.peekNextToken().type === lexer_1.TokenType.ParenthesisOpen) {
                    return this.parseFunction(lexer, errors, identifier);
                }
                else {
                    return this.parseVariable(lexer, errors, identifier);
                }
            case lexer_1.TokenType.ParenthesisOpen: return this.parseBracketedExpression(lexer, errors);
            case lexer_1.TokenType.Number: return this.parseNumber(lexer, errors);
            default: this.errorToken(errors, lexer_1.TokenType.Number, token);
        }
    };
    Parser.prototype.parseFunction = function (lexer, errors, name) {
        var functionName = name || this.parseDottedIdentifier(lexer, errors);
        this.expectAndConsume(lexer, errors, lexer_1.TokenType.ParenthesisOpen);
        var args = [];
        // Parse function arguments.
        var token = lexer.peekNextToken();
        if (token.type !== lexer_1.TokenType.ParenthesisClose) {
            args.push(this.parseExpression(lexer, errors));
            while (lexer.peekNextToken().type === lexer_1.TokenType.Comma) {
                lexer.getNextToken(); // Consume the comma.
                args.push(this.parseExpression(lexer, errors));
            }
        }
        this.expectAndConsume(lexer, errors, lexer_1.TokenType.ParenthesisClose);
        return function (s) {
            var resolvedArgs = args.map(function (a) { return a(s); });
            return s.resolve(functionName).apply(void 0, resolvedArgs);
        };
    };
    Parser.prototype.parseNumber = function (lexer, errors) {
        var numberToken = lexer.getNextToken();
        return function () { return Number(numberToken.value); };
    };
    Parser.prototype.parseVariable = function (lexer, errors, identifier) {
        var value = identifier || this.parseDottedIdentifier(lexer, errors);
        return function (s) { return s.resolve(value); };
    };
    Parser.prototype.parseBracketedExpression = function (lexer, errors) {
        lexer.getNextToken(); // Consume the opening bracket.
        var root = this.parseExpression(lexer, errors);
        this.expectAndConsume(lexer, errors, lexer_1.TokenType.ParenthesisClose);
        return root;
    };
    Parser.prototype.parseDottedIdentifier = function (lexer, errors) {
        var parts = [];
        parts.push(this.expectAndConsume(lexer, errors, lexer_1.TokenType.Identifier).value);
        while (lexer.peekNextToken().type === lexer_1.TokenType.Period) {
            lexer.getNextToken();
            parts.push(this.expectAndConsume(lexer, errors, lexer_1.TokenType.Identifier).value);
        }
        return parts;
    };
    return Parser;
}(parser_base_1.ParserBase));
exports.Parser = Parser;
//# sourceMappingURL=parser.js.map